<?php
namespace Models;

// модель для чеков
class Receipts extends \DB\SQL\Mapper {
	public $db, $f3, $fr;

	public function __construct($db, $f3) {
		parent::__construct($db, 'receipts' ); // подключаемся к таблице чеков
		$this->db=$db;
		$this->f3=$f3;
		$this->fr=new \Models\Fr($db, $f3);
	}

	// Информация об открываемой странице
	function page_data($page='sales') {
		$pages=array(
			'sales'=>array(
				'title'=>'Продажа',
				'inc'=>'receipts_sales.htm'
			),
			'return'=>array(
				'title'=>'Возврат',
				'inc'=>'receipts_return.htm'
			),
			'search'=>array(
				'title'=>'Поиск чеков',
				'inc'=>'receipts_search.htm'
			),
			'paymentmethods'=>array(
				'title'=>'Выбор способа оплаты',
				'inc'=>'receipts_paymentmethods.htm'
			),
			'show'=>array(
				'title'=>'Просмотр чека',
				'inc'=>'receipts_show.htm'
			),
			'change_price'=>array(
				'title'=>'Редактирование цены продажи',
				'inc'=>'receipts_change_price.htm'
			),
			'cashin'=>array(
				'title'=>'Внесение денег в кассу',
				'inc'=>'receipts_cashin.htm'
			),
			'cashout'=>array(
				'title'=>'Изъятие денег из кассы',
				'inc'=>'receipts_cashout.htm'
			),
			'reports'=>array(
				'title'=>'Смены и отчеты',
				'inc'=>'receipts_reports.htm'
			),
		);
		$pages[$page]['page']=$page;
		return $pages[$page];
	}

	// Выбираем чеки
	function select_receipts($query=null, $options=null) {
		if(!$query) {
			$query=array('id>?', 0); // чтобы хоть что-то вставить в фильтры
		}

		// формируем сортировку и лимиты
		$option='';
		if($options['order']) {
			$option.=' ORDER BY '.$options['order'];
		}
		if($options['limit']) {
			$option.=' LIMIT '.$options['limit'];
		}
		// собственно сам запрос
		$receipts=array();
		$sql="SELECT r.*, u.name, u.short_name, g.title FROM receipts AS r LEFT JOIN users AS u ON u.id=r.user_id LEFT JOIN usergroups AS g ON g.id=u.group_id WHERE ".$query[0].$option;
		$receipts=$this->db->exec($sql, $query[1]);
		$receiptlines=new \Models\Receiptlines($this->db, $this->f3);
		foreach ($receipts as $n => $r) {
			$receipts[$n]['receiptlines']=$receiptlines->select_lines(array('receipt_id=?', $r['id']));
			$receipts[$n]['paymentmethods']=json_decode($r['payment'], true);
		}
		return $receipts;
	}

	// Ищем чеки
	function search($id, $changes, $barcode, $dates, $type, $start=0, $limit=30) {
		$where='';
		$val=array();
		if($id) {
			$where='r.id=?';
			$val[]=$id;
		}
		if($changes) {
			$where='r.changes=?';
			$val[]=$changes;
		}
		if($dates) {
			if($where!='')$where.=' AND ';
			$where.='r.time>=? AND r.time<=?';
			$val[]=$dates[0];
			$val[]=$dates[1]+3600*24;
		}
		if($type) {
			if($where!='')$where.=' AND ';
			$where.='r.type=?';
			$val[]=$type;
		}
		if($barcode) {
			if($where!='') $where.=' AND ';
			$p=new \Models\Products($this->db);
			$products=$p->search($barcode, null, null, true);
			if($products['success']) {
				$product_id=$products['products'][0]['id'];
				$receiptlines=new \Models\Receiptlines($this->db, $this->f3);
				$lines=$receiptlines->select_lines(array('product_id=?', $product_id));
				if($lines) {
					$bwhere='';
					foreach ($lines as $l) {
						if($bwhere!='')$bwhere.=' OR ';
						$bwhere.=' r.id=? ';
						$val[]=$l['receipt_id'];
					}
					if($bwhere!='') {
						$where.=' ('.$bwhere.') ';
					}
				}
				else {
					$where.='r.id=?';
					$val[]=-1;
				}
			}
			else {
				$where.='r.id=?';
				$val[]=-1;
			}
		}
		if(!$where) {
			$where='r.id>?';
			$val[]=0;
		}
		$receipts=$this->select_receipts(array($where, $val), array('order'=>'r.id DESC', 'limit'=>$start.', '.$limit));
		$sql="SELECT COUNT(*) AS count FROM receipts AS r WHERE ".$where;
		$res=$this->db->exec($sql, $val);
		$count=$res[0]['count'];

		return array(
			'success'=>true,
			'count'=>$count,
			'receipts'=>$receipts
		);
	}

	// Пробиваем чек
	function ready($api_data, $change, $type, $paymentmethod, $client_sum, $sum, $lines, $client, $discount=0) {
		$this->reset();

		// Формируем данные оплаты
		$payment=array();
		foreach ($paymentmethod as $p_id) {
			$payment[]=array('method'=>$p_id, 'sum'=>$client_sum[$p_id]);
		}

		// формируем данные чека
		$receipt=array(
			'store'=>$api_data['store_id'],
			'rm'=>$api_data['rm_id'],
			'user_id'=>$this->f3->get('SESSION.user.id'),
			'changes'=>$change,
			'time'=>microtime(true),
			'type'=>$type,
			'payment'=>json_encode($payment),
			'sum'=>$sum,
			'client'=>$client
		);
		$c=new \Models\Controller($this->f3);
		$this->fr->ticket($receipt, $lines, $discount);
		$this->copyFrom($receipt);
		$this->insert(); // сохраняем
		$id=$this->get('_id'); // узнаем id
		if(!$id) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}

		return array(
			'success'=>true,
			'txt'=>'save_successful',
			'id'=>$id
		);
	}

	function tickets_end($id) {
		$this->fr->tickets_end($id);
	}

	// Удаление чека
	function delete_receipt($id) {
		$this->load(array('id=?', $id), array('limit'=>1));
		$this->erase();
		$this->db->exec("DELETE FROM receiptlines WHERE receipt_id=?", $id);
	}

	// Внесение и изъятие денег из кассы
	function cashin($api_data, $change, $pay, $sum) {
		$this->fr->cashin($change, $sum);
		//$sql="INSERT INTO cashin (time, changes, user_id, sum) VALUES (?, ?, ?, ?)";
		//$val=array(microtime(true), $change, $this->f3->get('SESSION.user.id'), $sum);
		//$res=$this->db->exec($sql, $val);
		if($sum>=0) $type=-1;
		else $type=-2;
		$receipt=array(
			'store'=>$api_data['store_id'],
			'rm'=>$api_data['rm_id'],
			'user_id'=>$this->f3->get('SESSION.user.id'),
			'changes'=>$change,
			'time'=>microtime(true),
			'type'=>$type,
			'payment'=>json_encode(array(array('method'=>$pay, 'sum'=>$sum))),
			'sum'=>$sum,
			'client'=>''
		);
		$this->reset();
		$this->copyFrom($receipt);
		$this->save();
		if($this->get('_id')) {
			return array(
				'success'=>true,
				'txt'=>'save_successful'
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Статистика по смене
	function change_statistic($change) {
		$receipts=$this->select_receipts(array('changes=?', $change));
		$report=array(
			'1'=>array('sum'=>0, 'count'=>0),
			'2'=>array('sum'=>0, 'count'=>0),
			'-1'=>array('sum'=>0, 'count'=>0),
			'-2'=>array('sum'=>0, 'count'=>0)
		);
		foreach ($receipts as $r) {
			$report[$r['type']]['count']+=1;
			$report[$r['type']]['sum']+=$r['sum'];
			$pays=json_decode($r['payment'], true);
			foreach ($pays as $p) {
				if($p['sum']>$r['sum']) {
					$p['sum']=$r['sum'];
				}
				$report[$r['type']]['pay'][$p['method']]+=$p['sum'];
			}
		}
		return $report;
	}
}